// HACore rel. 5

float4x4	WorldViewProjMatrix	: WORLDVIEWPROJECTION;
float4x4	WorldMatrix			: WORLD;
float4x4	ViewMatrix			: VIEW;
float4x4	ProjMatrix			: PROJECTION;
float3		CameraPosition		: POSITION		< string Object = "Camera"; >;

float4		MaterialDiffuse		: DIFFUSE		< string Object = "Geometry"; >;
float3		MaterialAmbient		: AMBIENT		< string Object = "Geometry"; >;
float3		MaterialEmmisive	: EMISSIVE		< string Object = "Geometry"; >;
float3		MaterialSpecular	: SPECULAR		< string Object = "Geometry"; >;
float		SpecularPower		: SPECULARPOWER	< string Object = "Geometry"; >;
float3		MaterialToon		: TOONCOLOR;
float4		EdgeColor			: EDGECOLOR;
float4		GroundShadowColor	: GROUNDSHADOWCOLOR;

float4		AddingTexture		: ADDINGTEXTURE;
float4		MultiplyTexture		: MULTIPLYINGTEXTURE;

#ifdef MIKUMIKUMOVING

float4		AddingSphere		: ADDINGSPHERE;
float4		MultiplySphere		: MULTIPLYINGSPHERE;

#else

float4		AddingSphere		: ADDINGSPHERETEXTURE;
float4		MultiplySphere		: MULTIPLYINGSPHERETEXTURE;

#endif

#ifdef USE_EXCELLENTSHADOW

float X_SHADOWPOWER = 1.0;
float PMD_SHADOWPOWER = 0.2;

shared texture2D ScreenShadowMapProcessed : RENDERCOLORTARGET
<
	float2 ViewPortRatio = { 1.0, 1.0 };
	int MipLevels = 1;
	string Format = "D3DFMT_R16F";
>;
sampler2D ScreenShadowMapProcessedSamp = sampler_state
{
	texture = <ScreenShadowMapProcessed>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = CLAMP;
	AddressV = CLAMP;
};

shared texture2D ExShadowSSAOMapOut : RENDERCOLORTARGET
<
	float2 ViewPortRatio = { 1.0, 1.0 };
	int MipLevels = 1;
	string Format = "R16F";
>;

sampler2D ExShadowSSAOMapSamp = sampler_state
{
	texture = <ExShadowSSAOMapOut>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = CLAMP;
	AddressV = CLAMP;
};

float2 ES_ViewportSize : VIEWPORTPIXELSIZE;
static float2 ES_ViewportOffset = (float2(0.5, 0.5) / ES_ViewportSize);

bool Exist_ExcellentShadow : CONTROLOBJECT < string name = "ExcellentShadow.x"; >;
bool Exist_ExShadowSSAO : CONTROLOBJECT < string name = "ExShadowSSAO.x"; >;
float ShadowRate : CONTROLOBJECT < string name = "ExcellentShadow.x"; string item = "Tr"; >;
float3 ES_CameraPos1 : POSITION < string Object = "Camera"; >;
float es_size0 : CONTROLOBJECT < string name = "ExcellentShadow.x"; string item = "Si"; >;
float4x4 es_mat1 : CONTROLOBJECT < string name = "ExcellentShadow.x"; >;

static float3 es_move1 = float3(es_mat1._41, es_mat1._42, es_mat1._43);
static float CameraDistance1 = length(ES_CameraPos1 - es_move1);

#endif

#ifdef MIKUMIKUMOVING

float		EdgeWidth			: EDGEWIDTH;

bool		LightEnables[MMM_LightCount]		: LIGHTENABLES;
float4x4	LightWVPMatrices[MMM_LightCount]	: LIGHTWVPMATRICES;
float3		LightDirection[MMM_LightCount]		: LIGHTDIRECTIONS;

float3		LightDiffuses[MMM_LightCount]		: LIGHTDIFFUSECOLORS;
float3		LightAmbients[MMM_LightCount]		: LIGHTAMBIENTCOLORS;
float3		LightSpeculars[MMM_LightCount]		: LIGHTSPECULARCOLORS;

float3		LightPositions[MMM_LightCount]		: LIGHTPOSITIONS;
float		LightZFars[MMM_LightCount]			: LIGHTZFARS;

static float4 DiffuseColor[MMM_LightCount] =
{
	MaterialDiffuse * float4(LightDiffuses[0], 1.0f),
	MaterialDiffuse * float4(LightDiffuses[1], 1.0f),
	MaterialDiffuse * float4(LightDiffuses[2], 1.0f)
};
static float3 AmbientColor[MMM_LightCount] =
{
	saturate(MaterialAmbient * LightAmbients[0] + MaterialEmmisive),
	saturate(MaterialAmbient * LightAmbients[1] + MaterialEmmisive),
	saturate(MaterialAmbient * LightAmbients[2] + MaterialEmmisive)
};
static float3 SpecularColor[MMM_LightCount] =
{
	MaterialSpecular * LightSpeculars[0],
	MaterialSpecular * LightSpeculars[1],
	MaterialSpecular * LightSpeculars[2]
};

#else

#define MMM_LightCount 1

float		EdgeWidth							= 1;

float4x4	LightWorldViewProjMatrix			: WORLDVIEWPROJECTION < string Object = "Light"; >;
float3		LightDirection0						: DIRECTION < string Object = "Light"; >;
float3		LightDiffuse						: DIFFUSE   < string Object = "Light"; >;
float3		LightAmbient						: AMBIENT   < string Object = "Light"; >;
float3		LightSpecular						: SPECULAR  < string Object = "Light"; >;

static bool		LightEnables[MMM_LightCount]		= { true };
static float4x4	LightWVPMatrices[MMM_LightCount]	= { LightWorldViewProjMatrix };
static float3	LightDirection[MMM_LightCount]		= { LightDirection0 };

static float3	LightDiffuses[MMM_LightCount]		= { LightDiffuse };
static float3	LightAmbients[MMM_LightCount]		= { LightAmbient };
static float3	LightSpeculars[MMM_LightCount]		= { LightSpecular };

static float4 DiffuseColor[MMM_LightCount] =
{
	MaterialDiffuse * float4(LightDiffuses[0], 1.0f)
};
static float3 AmbientColor[MMM_LightCount] =
{
	saturate(MaterialAmbient * LightAmbients[0] + MaterialEmmisive)
};
static float3 SpecularColor[MMM_LightCount] =
{
	MaterialSpecular * LightSpeculars[0]
};

#endif

bool	use_texture;		// eNX`gp
bool	use_spheremap;		// XtBA}bvgp
bool	use_toon;			// gD[`悩ǂ (ANZT: false, f: true)
bool	transp;				// tO
bool	spadd;				// XtBA}bvZtO

#ifdef MIKUMIKUMOVING

bool    usetoontexturemap;	// Toon eNX`gp邩ǂ (MMM)

#else

bool	parthf;				// ZtVhEtO (mode1: false, mode2: true)
#define SKII1	1500
#define SKII2	8000
#define Toon	3
sampler DefSampler : register(s0);

#endif

texture ObjectTexture : MATERIALTEXTURE;
sampler ObjTexSampler = sampler_state
{
	Texture = <ObjectTexture>;
	MinFilter = Anisotropic;
	MagFilter = Anisotropic;
	MipFilter = Linear;
	MaxAnisotropy = 16;
};
texture ObjectSphereMap : MATERIALSPHEREMAP;
sampler ObjSphereSampler = sampler_state
{
	Texture = <ObjectSphereMap>;
	MinFilter = Anisotropic;
	MagFilter = Anisotropic;
	MipFilter = Linear;
	MaxAnisotropy = 16;
};
texture ObjectToonTexture : MATERIALTOONTEXTURE;
sampler ObjToonSampler = sampler_state
{
	texture = <ObjectToonTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
	AddressU = Clamp;
	AddressV = Clamp;
};

///////////////////////////////////////////////////////////////////////////////////////////////

struct VS_OUTPUT
{
	float4 Pos				: POSITION;		// ˉeϊW
	float4 ZCalcTex			: TEXCOORD0;	// Z l
	float2 Tex				: TEXCOORD1;	// eNX`
	float3 Normal			: TEXCOORD2;	// @
	float3 Eye				: TEXCOORD3;	// JƂ̑Έʒu
	float2 SpTex			: TEXCOORD4;	// XtBA}bveNX`W
#ifdef MIKUMIKUMOVING
	float4 SS_UV1			: TEXCOORD5;	// ZtVhEeNX`W
	float4 SS_UV2			: TEXCOORD6;	// ZtVhEeNX`W
	float4 SS_UV3			: TEXCOORD7;	// ZtVhEeNX`W
#endif
#ifdef USE_EXCELLENTSHADOW
	float4 ScreenTex		: TEXCOORD8;
#endif
	float4 Color			: COLOR0;		// fBt[YF
};

#ifdef MIKUMIKUMOVING
VS_OUTPUT Basic_VS(MMM_SKINNING_INPUT IN, uniform bool useSelfShadow, uniform bool isEdge)
{
	MMM_SKINNING_OUTPUT SkinOut = MMM_SkinnedPositionNormal(IN.Pos, IN.Normal, IN.BlendWeight, IN.BlendIndices, IN.SdefC, IN.SdefR0, IN.SdefR1);
	float4 Pos = SkinOut.Position;
	float3 Normal = SkinOut.Normal;
	float2 Tex = IN.Tex;
#else
VS_OUTPUT Basic_VS(float4 Pos : POSITION, float3 Normal : NORMAL, float2 Tex : TEXCOORD0, uniform bool useSelfShadow, uniform bool isEdge)
{
#endif
	VS_OUTPUT Out = (VS_OUTPUT)0;
	
	Out.Eye.xyz = CameraPosition - mul(Pos, WorldMatrix);

	float dist = length(Out.Eye.xyz);

	if (isEdge)
	{
		float EdgeWeight = 0.001;

		if (!use_toon)
		{
			EdgeWeight *= 0.1;
			EdgeWidth = 1;
		}

#ifdef MIKUMIKUMOVING
		if (use_toon)
			EdgeWeight = IN.EdgeWeight;
#endif

#ifdef MIKUMIKUMOVING
		if (MMM_IsDinamicProjection)
			EdgeWeight *= MMM_GetDynamicFovEdgeRate(dist);
#endif

		Pos += float4(Normal, 0) * EdgeWeight * EdgeWidth * EdgeWidthRate * dist;
	}

#ifdef MIKUMIKUMOVING
	Out.Pos = mul(Pos, MMM_IsDinamicProjection ? mul(mul(WorldMatrix, ViewMatrix), MMM_DynamicFov(ProjMatrix, dist)) : WorldViewProjMatrix);
#else
	Out.Pos = mul(Pos, WorldViewProjMatrix);
#endif

	Out.Normal = normalize(mul(Normal, (float3x3)WorldMatrix));
	
	float3 color = 0;
	float3 ambient = 0;
	int count = 0;

	[unroll]
	for (int i = 0; i < MMM_LightCount; i++)
		if (LightEnables[i])
		{
			color += (float3(1, 1, 1) - color) * max(0, DiffuseColor[i] * dot(Out.Normal, -LightDirection[i]));
			ambient += AmbientColor[i];
			count++;
		}

	Out.Color.rgb = saturate(ambient / count + color);
	Out.Color.a = DiffuseColor[0].a;
	Out.Tex = Tex;
	
	if (use_spheremap)
		Out.SpTex.xy = mul(Out.Normal, (float3x3)ViewMatrix).xy * float2(0.5, -0.5) + float2(0.5, 0.5);

#ifdef MIKUMIKUMOVING
	if (!use_spheremap)
		Out.SpTex.xy = IN.AddUV1.xy;

	if (useSelfShadow)
	{
		float4 dpos = mul(Pos, WorldMatrix);

		Out.SS_UV1 = mul(dpos, LightWVPMatrices[0]);
		Out.SS_UV2 = mul(dpos, LightWVPMatrices[1]);
		Out.SS_UV3 = mul(dpos, LightWVPMatrices[2]);
		Out.SS_UV1.y = -Out.SS_UV1.y;
		Out.SS_UV2.y = -Out.SS_UV2.y;
		Out.SS_UV3.y = -Out.SS_UV3.y;
		Out.SS_UV1.z = length(LightPositions[0] - Pos) / LightZFars[0];
		Out.SS_UV2.z = length(LightPositions[1] - Pos) / LightZFars[1];
		Out.SS_UV3.z = length(LightPositions[2] - Pos) / LightZFars[2];
	}
#else
	Out.ZCalcTex = mul(Pos, LightWorldViewProjMatrix);
#endif

#ifdef USE_EXCELLENTSHADOW
	Out.ScreenTex = Out.Pos;
	Out.Pos.z -= max(0, (int)((CameraDistance1 - 6000) * 0.05));
#endif

	return Out;
}

float CalculateSelfShadowValue(VS_OUTPUT IN)
{
#ifdef USE_EXCELLENTSHADOW
	if (Exist_ExcellentShadow)
	{
		IN.ScreenTex.xyz /= IN.ScreenTex.w;

		float2 TransScreenTex = 0.5 + IN.ScreenTex * float2(0.5, -0.5) + ES_ViewportOffset;
		float ShadowMapVal = tex2D(ScreenShadowMapProcessedSamp, TransScreenTex).r;
		float ShadowColor = 0;
		float SSAOMapVal = 0;
		float3 lightdir = LightDirection[0];
		
		if (Exist_ExShadowSSAO)
			SSAOMapVal = tex2D(ExShadowSSAOMapSamp, TransScreenTex).r;
		
		if (use_toon)
		{
			ShadowMapVal = min(saturate(dot(IN.Normal, -lightdir) * 3), ShadowMapVal);
			ShadowColor *= 1 - (1 - ShadowRate) * PMD_SHADOWPOWER;
		}
		else
			ShadowColor *= 1 - (1 - ShadowRate) * X_SHADOWPOWER;

		float Color = 1;
		float ShadowColor2 = max(0, ShadowColor - (Color - ShadowColor + 0.3) * SSAOMapVal * 0.2);

		Color = lerp(Color, ShadowColor, saturate(SSAOMapVal * 0.4));
		Color = lerp(ShadowColor2, Color, ShadowMapVal);

		return Color;
	}
	else
#endif
	{
#ifdef MIKUMIKUMOVING
		return MMM_GetSelfShadowToonColor(float4(0, 0, 0, 1), IN.Normal, IN.SS_UV1, IN.SS_UV2, IN.SS_UV3, true, use_toon).r;
#else
		float4 uv = IN.ZCalcTex / IN.ZCalcTex.w;
		float2 TransTexCoord = 0.5 + uv * float2(0.5, -0.5);
		float comp = saturate(dot(IN.Normal, -LightDirection[0]) * 16 + 0.5);

		if (any(saturate(TransTexCoord) != TransTexCoord))
			return 1;
		else
		{
			comp = max(uv.z - tex2D(DefSampler, TransTexCoord).r, 0.0f);

			if (parthf)
				comp *= SKII2 * TransTexCoord.y - 0.3f;
			else
				comp *= SKII1 - 0.3f;

			comp = 1 - saturate(comp);

			return comp;
		}
#endif
	}
}

float4 Basic_PS(VS_OUTPUT IN, uniform bool useSelfShadow, uniform bool isEdge) : COLOR0
{
	float4 Color = IN.Color;
	float4 texColor = 1;
	float3 multiplySphereColor = 1;
	float3 addSphereColor = 0;
	BlendParameters ps = (BlendParameters)0;

	ps.ViewNormal = NanAsZero(max(dot(IN.Normal, normalize(IN.Eye.xyz)), 0));
	
	// texture
	if (use_texture)
	{
		texColor = tex2D(ObjTexSampler, IN.Tex);

		float texAlpha = MultiplyTexture.a + AddingTexture.a;

		texColor.rgb = (texColor.rgb * MultiplyTexture.rgb + AddingTexture.rgb) * texAlpha + (1.0 - texAlpha);
		Color *= texColor;
	}

	// spheremap
	if (use_spheremap)
		if (spadd)
			addSphereColor = tex2D(ObjSphereSampler, IN.SpTex.xy).rgb * MultiplySphere.rgb + AddingSphere.rgb;
		else
		{
			multiplySphereColor = tex2D(ObjSphereSampler, IN.SpTex.xy).rgb * MultiplySphere.rgb + AddingSphere.rgb;
			Color.rgb *= multiplySphereColor;
		}

	// shadow
	float shadowValue = 1;

#ifdef SHADE_TOONLESS
	use_toon = true;
#else
	#ifdef MIKUMIKUMOVING
	use_toon = use_toon ? usetoontexturemap : true;
	#else
	use_toon = use_toon ? MaterialToon != float3(1, 1, 1) : true;
	#endif
#endif

	if (useSelfShadow)
	{
		shadowValue = CalculateSelfShadowValue(IN);

		if (transp) Color.a = 0.5f;
	}

	// blending
	float3 diffuse = 0;
	float3 specular = 0;
	int lightCount = 0;

	[unroll]
	for (int i = 0; i < MMM_LightCount; i++)
		if (LightEnables[i])
		{
			ps.LightNormal = NanAsZero(use_toon ? saturate(min(shadowValue, dot(IN.Normal, -LightDirection[i]))) : 1);
			ps.HalfVectorNormal = NanAsZero(max(0, dot(normalize(normalize(IN.Eye.xyz) + -LightDirection[i]), normalize(IN.Normal))));
			ps.SpecularValue = NanAsZero(pow(ps.HalfVectorNormal, SpecularPower));
			ps.ToonValue = tex2D(ObjToonSampler, float2(0, 1 - ps.LightNormal)).rgb;

			if (isEdge)
				diffuse += BlendEdge(Color.rgb, ps);
			else
			{
#ifdef AMBIENT_AS_BASE
				float3 amb = AmbientColor[i] * texColor.rgb * multiplySphereColor;

	#ifdef AMBIENT_TOON_AS_BASE
				amb *= MaterialToon;
	#endif

				diffuse += amb + (BlendDiffuse(Color.rgb, ps) - amb) * (1 - amb);
#else
				diffuse += BlendDiffuse(Color.rgb, ps);
#endif
				specular += BlendSpecular(SpecularColor[i], ps);
				specular += BlendAddSphere(addSphereColor, ps);
			}

			lightCount++;
		}

	Color.rgb = saturate((diffuse + specular) / lightCount);
	
	return Color;
}

#ifdef HANDLE_EDGE

#define DRAW_EDGE_PASS(useSelfShadow) \
	pass DrawEdge \
	{ \
		CullMode = CW; \
		AlphaTestEnable = true; \
		VertexShader = compile vs_3_0 Basic_VS(useSelfShadow, true); \
		PixelShader  = compile ps_3_0 Basic_PS(useSelfShadow, true); \
	}

technique EdgeTec < string MMDPass = "edge"; >
{
}

#else

#define DRAW_EDGE_PASS(useSelfShadow)

#endif

#define DRAW_OBJECT_PASS(useSelfShadow) \
	pass DrawObject \
	{ \
		AlphaTestEnable = true; \
		VertexShader = compile vs_3_0 Basic_VS(useSelfShadow, false); \
		PixelShader  = compile ps_3_0 Basic_PS(useSelfShadow, false); \
	}

technique MainTec0 < string MMDPass = "object"; >
{
	DRAW_OBJECT_PASS(false)
	DRAW_EDGE_PASS(false)
}

technique MainTec1 < string MMDPass = "object_ss"; >
{
	DRAW_OBJECT_PASS(true)
	DRAW_EDGE_PASS(true)
}

///////////////////////////////////////////////////////////////////////////////////////////////
