#define DEFINE_TEXTURE_SAMPLER(textureName, samplerName, resourceName) \
	texture2D textureName < string ResourceName = resourceName; >; \
	sampler samplerName = sampler_state \
	{ \
		Texture = <textureName>; \
		Filter = Anisotropic; \
		AddressU = Clamp; \
		AddressV = Clamp; \
		MaxAnisotropy = 16; \
	};

static float NanAsZero(float value)
{
	return isnan(value) ? 0 : value;
}

static float3 GetHsvFromRgb(float3 rgb)
{
	float maxValue = max(max(rgb.r, rgb.g), rgb.b);
	float minValue = min(min(rgb.r, rgb.g), rgb.b);
	float delta = maxValue - minValue;
	float3 hsv = float3(0, 0, maxValue);

	if (delta != 0)
	{
		hsv.y = delta / maxValue;

		if (maxValue <= rgb.r)
			hsv.x = (rgb.g - rgb.b) / delta;
		else if (maxValue <= rgb.g)
			hsv.x = 2 + (rgb.b - rgb.r) / delta;
		else if (maxValue <= rgb.b)
			hsv.x = 4 + (rgb.r - rgb.g) / delta;

		hsv.x /= 6;
	}
	
	return hsv;
}

static float3 GetRgbFromHsv(float3 hsv)
{
	if (hsv.y == 0)
		return float3(hsv.z, hsv.z, hsv.z);
	else
	{
		if (hsv.x < 0)
			hsv.x += 1;
	
		hsv.yz = saturate(hsv.yz);

		float c = hsv.z * hsv.y;
		float x = c * (1 - abs(hsv.x * 6 % 2 - 1));
		float i = floor(hsv.x * 6) % 6;
		float m = hsv.z - c;

		c += m;
		x += m;

		if (i < 1)
			return float3(c, x, m);
		else if (i < 2)
			return float3(x, c, m);
		else if (i < 3)
			return float3(m, c, x);
		else if (i < 4)
			return float3(m, x, c);
		else if (i < 5)
			return float3(x, m, c);
		else
			return float3(c, m, x);
	}
}

struct BlendParameters
{
	float LightNormal;
	float ViewNormal;
	float HalfVectorNormal;
	float SpecularValue;
	float2 TextureCoordinate;
	float3 ToonValue;
};