#include "HAHead.hlsl"

#define BLENDDIFFUSE0TEXTUREFILENAME "diffuse_uwagi09.png"
//#define BLENDDIFFUSE1TEXTUREFILENAME "diffuse_uwagi01.png"
//#define BLENDDIFFUSE2TEXTUREFILENAME "DiffuseMulBySat.png"
//#define BLENDSPECULAR0TEXTUREFILENAME "SpecularMul.png"
//#define BLENDADDSPHERE0TEXTUREFILENAME "AddSphereRepl.png"
#define BLENDEDGE0TEXTUREFILENAME
#define USE_EXCELLENTSHADOW
//#define SHADE_TOONLESS
//#define AMBIENT_AS_BASE
//#define AMBIENT_TOON_AS_BASE
//#define HANDLE_EDGE
//#define FIX_LIGHTDIR
//#define FIX_SPHEREMAP
#ifdef BLENDDIFFUSE0TEXTUREFILENAME
DEFINE_TEXTURE_SAMPLER(BlendDiffuse0Texture, BlendDiffuse0TextureSampler, BLENDDIFFUSE0TEXTUREFILENAME)
#endif
#ifdef BLENDDIFFUSE1TEXTUREFILENAME
DEFINE_TEXTURE_SAMPLER(BlendDiffuse1Texture, BlendDiffuse1TextureSampler, BLENDDIFFUSE1TEXTUREFILENAME)
#endif
#ifdef BLENDDIFFUSE2TEXTUREFILENAME
DEFINE_TEXTURE_SAMPLER(BlendDiffuse2Texture, BlendDiffuse2TextureSampler, BLENDDIFFUSE2TEXTUREFILENAME)
#endif
#ifdef BLENDSPECULAR0TEXTUREFILENAME
DEFINE_TEXTURE_SAMPLER(BlendSpecular0Texture, BlendSpecular0TextureSampler, BLENDSPECULAR0TEXTUREFILENAME)
#endif
#ifdef BLENDADDSPHERE0TEXTUREFILENAME
DEFINE_TEXTURE_SAMPLER(BlendAddSphere0Texture, BlendAddSphere0TextureSampler, BLENDADDSPHERE0TEXTUREFILENAME)
#endif

float EdgeWidthRate
<
	string UIName = "GbW";
	float UIMin = 0;
	float UIMax = 2;
> = 0.7;

float3 BlendDiffuse0(float3 target, BlendParameters parameters)
{
	#ifdef BLENDDIFFUSE0TEXTUREFILENAME
	float4 value = tex2D(BlendDiffuse0TextureSampler, float2(parameters.HalfVectorNormal, 0));
	
	target.r = lerp(target.r, value.r, value.a);
	target.g = lerp(target.g, value.g, value.a);
	target.b = lerp(target.b, value.b, value.a);
	#endif
	
	return target;
}

float3 BlendDiffuse1(float3 target, BlendParameters parameters)
{
	#ifdef BLENDDIFFUSE1TEXTUREFILENAME
	float4 value = tex2D(BlendDiffuse1TextureSampler, float2(parameters.LightNormal, 0));
	
	target.r = lerp(target.r, target.r * value.r, value.a);
	target.g = lerp(target.g, target.g * value.g, value.a);
	target.b = lerp(target.b, target.b * value.b, value.a);
	#endif
	
	return target;
}

float3 BlendDiffuse2(float3 target, BlendParameters parameters)
{
	#ifdef BLENDDIFFUSE2TEXTUREFILENAME
	float3 targetHsv = GetHsvFromRgb(target);
	float4 value = tex2D(BlendDiffuse2TextureSampler, float2(parameters.LightNormal, targetHsv.y));
	
	target.r = lerp(target.r, target.r * value.r, value.a);
	target.g = lerp(target.g, target.g * value.g, value.a);
	target.b = lerp(target.b, target.b * value.b, value.a);
	#endif
	
	return target;
}

float3 BlendDiffuse(float3 color, BlendParameters parameters)
{
	color = BlendDiffuse0(color, parameters);
	color = BlendDiffuse1(color, parameters);
	color = BlendDiffuse2(color, parameters);

	return color;
}

float3 BlendSpecular0(float3 target, BlendParameters parameters)
{
	#ifdef BLENDSPECULAR0TEXTUREFILENAME
	float4 value = tex2D(BlendSpecular0TextureSampler, float2(parameters.SpecularValue, 0));
	
	target.r = lerp(target.r, target.r * value.r, value.a);
	target.g = lerp(target.g, target.g * value.g, value.a);
	target.b = lerp(target.b, target.b * value.b, value.a);
	#endif
	
	return target;
}

float3 BlendSpecular(float3 color, BlendParameters parameters)
{
	color = BlendSpecular0(color, parameters);

	return color;
}

float3 BlendAddSphere0(float3 target, BlendParameters parameters)
{
	#ifdef BLENDADDSPHERE0TEXTUREFILENAME
	float3 targetHsv = GetHsvFromRgb(target);
	float4 value = tex2D(BlendAddSphere0TextureSampler, float2(targetHsv.z, 0));
	
	targetHsv.z = lerp(targetHsv.z, value.b, value.a);
	target = GetRgbFromHsv(targetHsv);
	#endif
	
	return target;
}

float3 BlendAddSphere(float3 color, BlendParameters parameters)
{
	color = BlendAddSphere0(color, parameters);

	return color;
}

float3 BlendEdge0(float3 target, BlendParameters parameters)
{
	#ifdef BLENDEDGE0TEXTUREFILENAME
	float4 value = float4(0, 0, 0, 1);
	
	target.r = lerp(target.r, target.r * value.r, value.a);
	target.g = lerp(target.g, target.g * value.g, value.a);
	target.b = lerp(target.b, target.b * value.b, value.a);
	#endif
	
	return target;
}

float3 BlendEdge(float3 color, BlendParameters parameters)
{
	color = BlendEdge0(color, parameters);

	return color;
}

#include "HACore.hlsl"

// HAGenerator Document
// HA2 QA5FZmZlY3REb2N1bWVudAQVQ29weUNvcmVUb0Rlc3RpbmF0aW9umARUcnVlBAlFZGdlV2lkdGiY
// HA2 AzAuNwQRRml4TGlnaHREaXJlY3Rpb26YBUZhbHNlBAxGaXhTcGhlcmVNYXCYBUZhbHNlBApIYW5k
// HA2 bGVFZGdlmAVGYWxzZQQQU2V0QW1iaWVudEFzQmFzZZgFRmFsc2UEGFNldEFtYmllbnRBc0Jhc2VX
// HA2 aXRoVG9vbpgFRmFsc2UEDVNoYWRlVG9vbmxlc3OYBUZhbHNlBBJVc2VFeGNlbGxlbnRTaGFkb3eY
// HA2 BFRydWUIOWNsci1uYW1lc3BhY2U6TGluZWFyc3Rhci5IQUdlbmVyYXRvcjthc3NlbWJseT1IQUdl
// HA2 bmVyYXRvcgkBeCxodHRwOi8vc2NoZW1hcy5taWNyb3NvZnQuY29tL3dpbmZ4LzIwMDYveGFtbEAf
// HA2 RWZmZWN0RG9jdW1lbnQuQWRkU3BoZXJlRW50cmllc0AKQmxlbmRFbnRyeQQJSXNFbmFibGVkmAVG
// HA2 YWxzZQQKSXNTcGVjdWxhcpgEVHJ1ZQQKUGFyYW1ldGVyWJgFVmFsdWUEClBhcmFtZXRlclmYBE5v
// HA2 bmVAEkJsZW5kRW50cnkuQWN0aW9uc3UFQXJyYXkEBFR5cGWYC0JsZW5kQWN0aW9uQAtCbGVuZEFj
// HA2 dGlvbgQES2luZJgETm9uZQQQUGFyYW1ldGVyQ2hhbm5lbJgDUmVkBA1UYXJnZXRDaGFubmVsmANI
// HA2 dWUBQAtCbGVuZEFjdGlvbgQES2luZJgETm9uZQQQUGFyYW1ldGVyQ2hhbm5lbJgFR3JlZW4EDVRh
// HA2 cmdldENoYW5uZWyYClNhdHVyYXRpb24BQAtCbGVuZEFjdGlvbgQES2luZJgHUmVwbGFjZQQQUGFy
// HA2 YW1ldGVyQ2hhbm5lbJgEQmx1ZQQNVGFyZ2V0Q2hhbm5lbJgFVmFsdWUBAQFAEkJsZW5kRW50cnku
// HA2 VGV4dHVyZUAHVGV4dHVyZQQETmFtZZgIe3g6TnVsbH0ECEZpbGVOYW1lmBFBZGRTcGhlcmVSZXBs
// HA2 LnBuZwQES2luZJgDUmdiBAxWYXJpYWJsZU5hbWWYFkJsZW5kQWRkU3BoZXJlMFRleHR1cmUBAQEB
// HA2 QB1FZmZlY3REb2N1bWVudC5EaWZmdXNlRW50cmllc0AKQmxlbmRFbnRyeQQJSXNFbmFibGVkmARU
// HA2 cnVlBApJc1NwZWN1bGFymAVGYWxzZQQKUGFyYW1ldGVyWJgQSGFsZlZlY3Rvck5vcm1hbAQKUGFy
// HA2 YW1ldGVyWZgETm9uZUASQmxlbmRFbnRyeS5BY3Rpb25zdQVBcnJheQQEVHlwZZgLQmxlbmRBY3Rp
// HA2 b25AC0JsZW5kQWN0aW9uBARLaW5kmAdSZXBsYWNlBBBQYXJhbWV0ZXJDaGFubmVsmANSZWQEDVRh
// HA2 cmdldENoYW5uZWyYA1JlZAFAC0JsZW5kQWN0aW9uBARLaW5kmAdSZXBsYWNlBBBQYXJhbWV0ZXJD
// HA2 aGFubmVsmAVHcmVlbgQNVGFyZ2V0Q2hhbm5lbJgFR3JlZW4BQAtCbGVuZEFjdGlvbgQES2luZJgH
// HA2 UmVwbGFjZQQQUGFyYW1ldGVyQ2hhbm5lbJgEQmx1ZQQNVGFyZ2V0Q2hhbm5lbJgEQmx1ZQEBAUAS
// HA2 QmxlbmRFbnRyeS5UZXh0dXJlQAdUZXh0dXJlBAROYW1lmAh7eDpOdWxsfQQIRmlsZU5hbWWYE2Rp
// HA2 ZmZ1c2VfdXdhZ2kwOS5wbmcEBEtpbmSYA1JnYgQMVmFyaWFibGVOYW1lmBRCbGVuZERpZmZ1c2Uw
// HA2 VGV4dHVyZQEBAUAKQmxlbmRFbnRyeQQJSXNFbmFibGVkmAVGYWxzZQQKSXNTcGVjdWxhcpgFRmFs
// HA2 c2UEClBhcmFtZXRlcliYC0xpZ2h0Tm9ybWFsBApQYXJhbWV0ZXJZmAROb25lQBJCbGVuZEVudHJ5
// HA2 LkFjdGlvbnN1BUFycmF5BARUeXBlmAtCbGVuZEFjdGlvbkALQmxlbmRBY3Rpb24EBEtpbmSYCE11
// HA2 bHRpcGx5BBBQYXJhbWV0ZXJDaGFubmVsmANSZWQEDVRhcmdldENoYW5uZWyYA1JlZAFAC0JsZW5k
// HA2 QWN0aW9uBARLaW5kmAhNdWx0aXBseQQQUGFyYW1ldGVyQ2hhbm5lbJgFR3JlZW4EDVRhcmdldENo
// HA2 YW5uZWyYBUdyZWVuAUALQmxlbmRBY3Rpb24EBEtpbmSYCE11bHRpcGx5BBBQYXJhbWV0ZXJDaGFu
// HA2 bmVsmARCbHVlBA1UYXJnZXRDaGFubmVsmARCbHVlAQEBQBJCbGVuZEVudHJ5LlRleHR1cmVAB1Rl
// HA2 eHR1cmUEBE5hbWWYCHt4Ok51bGx9BAhGaWxlTmFtZZgTZGlmZnVzZV91d2FnaTAxLnBuZwQES2lu
// HA2 ZJgDUmdiBAxWYXJpYWJsZU5hbWWYFEJsZW5kRGlmZnVzZTFUZXh0dXJlAQEBQApCbGVuZEVudHJ5
// HA2 BAlJc0VuYWJsZWSYBUZhbHNlBApJc1NwZWN1bGFymAVGYWxzZQQKUGFyYW1ldGVyWJgLTGlnaHRO
// HA2 b3JtYWwEClBhcmFtZXRlclmYClNhdHVyYXRpb25AEkJsZW5kRW50cnkuQWN0aW9uc3UFQXJyYXkE
// HA2 BFR5cGWYC0JsZW5kQWN0aW9uQAtCbGVuZEFjdGlvbgQES2luZJgITXVsdGlwbHkEEFBhcmFtZXRl
// HA2 ckNoYW5uZWyYA1JlZAQNVGFyZ2V0Q2hhbm5lbJgDUmVkAUALQmxlbmRBY3Rpb24EBEtpbmSYCE11
// HA2 bHRpcGx5BBBQYXJhbWV0ZXJDaGFubmVsmAVHcmVlbgQNVGFyZ2V0Q2hhbm5lbJgFR3JlZW4BQAtC
// HA2 bGVuZEFjdGlvbgQES2luZJgITXVsdGlwbHkEEFBhcmFtZXRlckNoYW5uZWyYBEJsdWUEDVRhcmdl
// HA2 dENoYW5uZWyYBEJsdWUBAQFAEkJsZW5kRW50cnkuVGV4dHVyZUAHVGV4dHVyZQQETmFtZZgIe3g6
// HA2 TnVsbH0ECEZpbGVOYW1lmBNEaWZmdXNlTXVsQnlTYXQucG5nBARLaW5kmANSZ2IEDFZhcmlhYmxl
// HA2 TmFtZZgUQmxlbmREaWZmdXNlMlRleHR1cmUBAQEBQBpFZmZlY3REb2N1bWVudC5FZGdlRW50cmll
// HA2 c0AKQmxlbmRFbnRyeQQJSXNFbmFibGVkmARUcnVlBApJc1NwZWN1bGFymAVGYWxzZQQKUGFyYW1l
// HA2 dGVyWJgETm9uZQQKUGFyYW1ldGVyWZgETm9uZUASQmxlbmRFbnRyeS5BY3Rpb25zdQVBcnJheQQE
// HA2 VHlwZZgLQmxlbmRBY3Rpb25AC0JsZW5kQWN0aW9uBARLaW5kmAhNdWx0aXBseQQQUGFyYW1ldGVy
// HA2 Q2hhbm5lbJgDUmVkBA1UYXJnZXRDaGFubmVsmANSZWQBQAtCbGVuZEFjdGlvbgQES2luZJgITXVs
// HA2 dGlwbHkEEFBhcmFtZXRlckNoYW5uZWyYBUdyZWVuBA1UYXJnZXRDaGFubmVsmAVHcmVlbgFAC0Js
// HA2 ZW5kQWN0aW9uBARLaW5kmAhNdWx0aXBseQQQUGFyYW1ldGVyQ2hhbm5lbJgEQmx1ZQQNVGFyZ2V0
// HA2 Q2hhbm5lbJgEQmx1ZQEBAUASQmxlbmRFbnRyeS5UZXh0dXJlQAdUZXh0dXJlBAhGaWxlTmFtZZgI
// HA2 e3g6TnVsbH0EBE5hbWWYCHt4Ok51bGx9BARLaW5kmANSZ2IEDFZhcmlhYmxlTmFtZZgRQmxlbmRF
// HA2 ZGdlMFRleHR1cmUBAQEBQB5FZmZlY3REb2N1bWVudC5TcGVjdWxhckVudHJpZXNACkJsZW5kRW50
// HA2 cnkECUlzRW5hYmxlZJgFRmFsc2UECklzU3BlY3VsYXKYBFRydWUEClBhcmFtZXRlcliYDVNwZWN1
// HA2 bGFyVmFsdWUEClBhcmFtZXRlclmYBE5vbmVAEkJsZW5kRW50cnkuQWN0aW9uc3UFQXJyYXkEBFR5
// HA2 cGWYC0JsZW5kQWN0aW9uQAtCbGVuZEFjdGlvbgQES2luZJgITXVsdGlwbHkEEFBhcmFtZXRlckNo
// HA2 YW5uZWyYA1JlZAQNVGFyZ2V0Q2hhbm5lbJgDUmVkAUALQmxlbmRBY3Rpb24EBEtpbmSYCE11bHRp
// HA2 cGx5BBBQYXJhbWV0ZXJDaGFubmVsmAVHcmVlbgQNVGFyZ2V0Q2hhbm5lbJgFR3JlZW4BQAtCbGVu
// HA2 ZEFjdGlvbgQES2luZJgITXVsdGlwbHkEEFBhcmFtZXRlckNoYW5uZWyYBEJsdWUEDVRhcmdldENo
// HA2 YW5uZWyYBEJsdWUBAQFAEkJsZW5kRW50cnkuVGV4dHVyZUAHVGV4dHVyZQQETmFtZZgIe3g6TnVs
// HA2 bH0ECEZpbGVOYW1lmA9TcGVjdWxhck11bC5wbmcEBEtpbmSYA1JnYgQMVmFyaWFibGVOYW1lmBVC
// HA2 bGVuZFNwZWN1bGFyMFRleHR1cmUBAQEBAQ==